@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: TestRunner.h 2326 2006-12-29 17:46:23Z svn $
+ * $Id: TestRunner.h 2388 2007-03-10 11:34:50Z svn $
  *****************************************************************************/
 
 #ifndef TEST_RUNNER_H
@@ -163,21 +163,6 @@ void show_item(const char *key, double value, const char *unit)
     cerr.setf(ios::fixed, ios::floatfield);
     cerr << value << unit << endl;
 }
-/**
- * ファイル/ディレクトリが存在するかどうかを調べます．
- *
- * @param[in] pool プール
- * @param[in] path パス
- */
-bool is_exist(apr_pool_t *pool, const char *path)
-{
-    apr_finfo_t info;
-    apr_status_t status;
-
-    status = apr_stat(&info, path, APR_FINFO_MIN, pool);
-
-    return !APR_STATUS_IS_ENOENT(status);
-}
 
 /**
  * アドレスを適当な構造体に変換します．
