@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: UploadItemReader.cpp 2077 2006-11-25 11:38:44Z svn $
+ * $Id: UploadItemReader.cpp 2144 2006-11-29 14:54:20Z svn $
  *****************************************************************************/
 
 #include "Environment.h"
@@ -39,7 +39,7 @@
 #include "Macro.h"
 #include "SourceInfo.h"
 
-SOURCE_INFO_ADD("$Id: UploadItemReader.cpp 2077 2006-11-25 11:38:44Z svn $");
+SOURCE_INFO_ADD("$Id: UploadItemReader.cpp 2144 2006-11-29 14:54:20Z svn $");
 
 /******************************************************************************
  * public メソッド
@@ -80,7 +80,7 @@ void UploadItemReader::read(apr_size_t item_id, UploadItem **bbs_thread,
     *bbs_thread = AS_UITEM(uitem_file_map->mm);
 }
 
-bool UploadItemReader::is_exist(apr_size_t item_id) const
+bool UploadItemReader::is_exist_item(apr_size_t item_id) const
 {
     const char *uitem_file_path;
     apr_finfo_t info;
@@ -92,6 +92,18 @@ bool UploadItemReader::is_exist(apr_size_t item_id) const
                                           APR_FINFO_MIN, temp_pool.get()));
 }
 
+bool UploadItemReader::is_exist_thumbnail(apr_size_t item_id) const
+{
+    const char *thumbnail_file_path;
+    apr_finfo_t info;
+    TemporaryPool temp_pool;
+
+    thumbnail_file_path = get_thumb_path(temp_pool.get(), item_id);
+
+    return !APR_STATUS_IS_ENOENT(apr_stat(&info, thumbnail_file_path,
+                                          APR_FINFO_MIN, temp_pool.get()));
+}
+
 UploadItem *UploadItemReader::read(apr_pool_t *pool, const char *dir_path,
                                  apr_size_t item_id)
 {
