@@ -22,7 +22,7 @@
  * 3. This notice may not be removed or altered from any source
  *    distribution.
  *
- * $Id: mod_uploader.cpp 2291 2006-12-22 14:07:00Z svn $
+ * $Id: mod_uploader.cpp 2422 2007-05-13 14:50:22Z svn $
  *****************************************************************************/
 
 /**
@@ -40,6 +40,7 @@
 #include "http_request.h"
 
 #include "apr_pools.h"
+#include "apr_tables.h"
 #include "apr_strings.h"
 
 #include "mod_uploader.h"
@@ -58,14 +59,15 @@
 
 using namespace std;
 
-SOURCE_INFO_ADD("$Id: mod_uploader.cpp 2291 2006-12-22 14:07:00Z svn $");
+SOURCE_INFO_ADD("$Id: mod_uploader.cpp 2422 2007-05-13 14:50:22Z svn $");
 
 static const char *HANDLER_NAME = "uploader";
+static apr_array_header_t *uploader_config_array = NULL;
 
 #define DIRECTIVE_ENTRY(directive, member)                              \
     AP_INIT_TAKE1(UploaderConfig::directive.param,                      \
                   reinterpret_cast<const char*(*)()>(set_ ## member),   \
-                  NULL, RSRC_CONF, UploaderConfig::directive.desc)
+                  NULL, ACCESS_CONF, UploaderConfig::directive.desc)
 
 #define AS_UCONFIG(pointer)         reinterpret_cast<UploaderConfig *>(pointer)
 
@@ -75,8 +77,7 @@ static const char *HANDLER_NAME = "uploader";
     {                                                                   \
         UploaderConfig *config;                                         \
                                                                         \
-        config = AS_UCONFIG(ap_get_module_config(parms->server->module_config, \
-                                                 &uploader_module));    \
+        config = AS_UCONFIG(mconfig);                                   \
         config->member = value;                                         \
                                                                         \
         return NULL;                                                    \
@@ -87,20 +88,26 @@ static const char *HANDLER_NAME = "uploader";
     {                                                                   \
         UploaderConfig *config;                                         \
                                                                         \
-        config = AS_UCONFIG(ap_get_module_config(parms->server->module_config, \
-                                                 &uploader_module));    \
+        config = AS_UCONFIG(mconfig);                                   \
         config->set_template(UploaderConfig::type, tmpl_file_path);     \
                                                                         \
         return NULL;                                                    \
     }
 
-UploaderConfig *get_config(request_rec *r)
+static UploaderConfig *uploader_get_dir_config(request_rec *r)
 {
-    return AS_UCONFIG(ap_get_module_config(r->server->module_config,
+    return AS_UCONFIG(ap_get_module_config(r->per_dir_config,
                                            &uploader_module));
 }
 
-static apr_status_t finalize_uploader_config(void *data)
+static apr_status_t uploader_finalize_server_config(void *data)
+{
+    uploader_config_array = NULL;
+
+    return APR_SUCCESS;
+}
+
+static apr_status_t uploader_finalize_dir_config(void *data)
 {
     UploaderConfig *config;
 
@@ -111,12 +118,27 @@ static apr_status_t finalize_uploader_config(void *data)
 }
 
 static void *uploader_create_server_config(apr_pool_t *p, server_rec *s)
+{
+    if (uploader_config_array == NULL) {
+        uploader_config_array = apr_array_make(p, 4, sizeof(UploaderConfig *));
+
+        apr_pool_cleanup_register(p, NULL, uploader_finalize_server_config,
+                                  apr_pool_cleanup_null);
+    }
+
+    return NULL;
+}
+
+static void *uploader_create_dir_config(apr_pool_t *p, char *dirspec)
 {
     UploaderConfig *config;
 
-    APR_PCALLOC(config, UploaderConfig *, s->process->pool, sizeof(UploaderConfig));
+    APR_PCALLOC(config, UploaderConfig *, p, sizeof(UploaderConfig));
 
-    new(config) UploaderConfig(s->process->pool);
+    new(config) UploaderConfig(p);
+
+    memcpy(apr_array_push(uploader_config_array), &config,
+           sizeof(UploaderConfig *));
 
     return config;
 }
@@ -139,20 +161,25 @@ static bool get_apache_debug_mode()
 
 static int uploader_post_config_server(apr_pool_t *p, server_rec *s)
 {
+    UploaderConfig **configs;
     UploaderConfig *config;
 
-    config = AS_UCONFIG(ap_get_module_config(s->module_config,
-                                             &uploader_module));
-    try {
-        config->init();
-        config->is_debug_mode = get_apache_debug_mode();
+    configs = (UploaderConfig **)uploader_config_array->elts;
 
-        apr_pool_cleanup_register(p, config, finalize_uploader_config,
-                                  apr_pool_cleanup_null);
-    } catch(const char *message) {
-        logger.error(__FILE__, __LINE__, s,
-                     "(%s) %s", "UploaderConfig::init", message);
-        return HTTP_INTERNAL_SERVER_ERROR;
+    for (int i = 0; i < uploader_config_array->nelts; i++) {
+        config = configs[i];
+
+        try {
+            config->init();
+            config->is_debug_mode = get_apache_debug_mode();
+
+            apr_pool_cleanup_register(p, config, uploader_finalize_dir_config,
+                                      apr_pool_cleanup_null);
+        } catch(const char *message) {
+            logger.error(__FILE__, __LINE__, s,
+                         "(%s) %s", "UploaderConfig::init", message);
+            return HTTP_INTERNAL_SERVER_ERROR;
+        }
     }
 
     return OK;
@@ -174,12 +201,10 @@ static int uploader_post_config(apr_pool_t *pconf, apr_pool_t *plog,
         return OK;
     }
 
-    do {
-        status = uploader_post_config_server(pconf, s);
-        if (status != OK) {
-            return status;
-        }
-    } while ((s = s->next) != NULL);
+    status = uploader_post_config_server(pconf, s);
+    if (status != OK) {
+        return status;
+    }
 
 #ifndef WIN32
     // Windows でこれをするとサービス名が変わってしまうらしい
@@ -191,26 +216,29 @@ static int uploader_post_config(apr_pool_t *pconf, apr_pool_t *plog,
     return OK;
 }
 
-static void uploader_child_init_server(apr_pool_t *pool, server_rec *server)
+static void uploader_child_init_server(apr_pool_t *pool, server_rec *s)
 {
+    UploaderConfig **configs;
     UploaderConfig *config;
 
-    config = AS_UCONFIG(ap_get_module_config(server->module_config,
-                                             &uploader_module));
-    try {
-        config->child_init();
-    } catch(const char *message) {
-        logger.error(__FILE__, __LINE__, server,
-                     "(%s) %s: %s", "UploaderConfig::child_init", message,
-                     get_last_error_message(pool));
+    configs = (UploaderConfig **)uploader_config_array->elts;
+
+    for (int i = 0; i < uploader_config_array->nelts; i++) {
+        config = configs[i];
+
+        try {
+            config->child_init();
+        } catch(const char *message) {
+            logger.error(__FILE__, __LINE__, s,
+                         "(%s) %s: %s", "UploaderConfig::child_init", message,
+                         get_last_error_message(pool));
+        }
     }
 }
 
 static void uploader_child_init(apr_pool_t *p, server_rec *s)
 {
-    do {
-        uploader_child_init_server(p, s);
-    } while ((s = s->next) != NULL);
+    uploader_child_init_server(p, s);
 
     logger.info(__FILE__, __LINE__, s, "SUCCESS: %s", "child init");
 }
@@ -227,7 +255,7 @@ static int uploader_handler(request_rec *r)
         return DECLINED;
     }
 
-    config = get_config(r);
+    config = uploader_get_dir_config(r);
 
     if (UNLIKELY(!config->is_active())) {
         return DECLINED;
@@ -293,7 +321,7 @@ static const command_rec uploader_cmds[] = {
 extern "C" {
 module EXPORT AP_MODULE_DECLARE_DATA uploader_module = {
     STANDARD20_MODULE_STUFF,
-    NULL,
+    uploader_create_dir_config,
     NULL,
     uploader_create_server_config,
     NULL,
